"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useAuth = exports.AuthContextProvider = exports.AuthContext = void 0;
const react_1 = __importStar(require("react"));
const react_async_hook_1 = require("react-async-hook");
const react_router_1 = require("react-router");
const api_1 = __importStar(require("../../../helpers/api"));
const AuthContext = (0, react_1.createContext)({});
exports.AuthContext = AuthContext;
const AuthContextProvider = ({ children }) => {
    const [user, setUser] = (0, react_1.useState)(undefined);
    const [loading, setLoading] = (0, react_1.useState)(true);
    const history = (0, react_router_1.useHistory)();
    const [authStatus, setAuthStatus] = (0, react_1.useState)('unauthenticated');
    const logIn = (0, api_1.useApiCallback)(({ email, password }) => __awaiter(void 0, void 0, void 0, function* () {
        var _a;
        const response = yield api_1.default.post('/login', { email, password });
        setUser((_a = response === null || response === void 0 ? void 0 : response.data) === null || _a === void 0 ? void 0 : _a.user);
        history.replace('/');
        return response;
    }));
    (0, react_1.useEffect)(() => {
        setAuthStatus(user ? 'authenticated' : 'unauthenticated');
    }, [user]);
    const logOut = (0, api_1.useApiCallback)(() => __awaiter(void 0, void 0, void 0, function* () {
        const response = yield api_1.default.post('/logout');
        setUser(undefined);
        history.replace('/login');
        return response;
    }));
    const loggedIn = (0, react_async_hook_1.useAsyncCallback)(() => __awaiter(void 0, void 0, void 0, function* () {
        try {
            const res = yield api_1.default.get('/loggedIn');
            setUser(res === null || res === void 0 ? void 0 : res.data);
            setAuthStatus('authenticated');
        }
        catch (e) {
            const { status } = e.response;
            if (user) {
                setUser(undefined);
            }
            if (status === 401) {
                setAuthStatus('unauthenticated');
            }
            else if (status === 400) {
                setAuthStatus('needs-init');
            }
        }
    }));
    const forgot = (0, api_1.useApiCallback)(({ email }) => __awaiter(void 0, void 0, void 0, function* () { return api_1.default.post('/forgot', { email }); }));
    const forgotVerify = (0, react_async_hook_1.useAsyncCallback)(({ token, password }) => __awaiter(void 0, void 0, void 0, function* () { return api_1.default.post('/forgot/verify', { token, password }); }));
    const init = (0, api_1.useApiCallback)((data) => __awaiter(void 0, void 0, void 0, function* () {
        var _b;
        const res = yield api_1.default.post('/init', data);
        setUser((_b = res === null || res === void 0 ? void 0 : res.data) === null || _b === void 0 ? void 0 : _b.user);
        history.replace('/');
        return res;
    }));
    const load = (0, react_async_hook_1.useAsyncCallback)(() => __awaiter(void 0, void 0, void 0, function* () {
        setLoading(true);
        yield Promise.all([loggedIn.execute()]);
        setLoading(false);
    }));
    const updatePassword = (0, api_1.useApiCallback)(({ currentPassword, password, confirmPassword }) => __awaiter(void 0, void 0, void 0, function* () {
        return api_1.default.post('/profile/change-password', {
            currentPassword,
            password,
            confirmPassword,
        });
    }));
    (0, react_1.useEffect)(() => {
        load.execute();
        api_1.default.interceptors.response.use(undefined, (err) => __awaiter(void 0, void 0, void 0, function* () {
            const { response } = err;
            if (response.status === 401) {
                setUser(undefined);
                history.replace('/login');
            }
            return Promise.reject(err);
        }));
    }, []);
    const filterPermissions = (array) => {
        const permissions = ((user === null || user === void 0 ? void 0 : user.groups) || []).reduce((acc, group) => {
            return [...(acc || []), ...((group === null || group === void 0 ? void 0 : group.permissions) || [])];
        }, []);
        const find = (val) => !!permissions.find((permission) => permission === val);
        if (find('all'))
            return array;
        return array.filter((item) => {
            var _a, _b;
            return !((_a = item) === null || _a === void 0 ? void 0 : _a.permissions) ||
                (((_b = item) === null || _b === void 0 ? void 0 : _b.permissions) || []).some(find);
        });
    };
    const hasPermission = (array) => {
        const permissions = ((user === null || user === void 0 ? void 0 : user.groups) || []).reduce((acc, group) => {
            return [...(acc || []), ...((group === null || group === void 0 ? void 0 : group.permissions) || [])];
        }, []);
        const find = (val) => !!permissions.find((permission) => permission === val);
        return find('all') || (array || []).some((permission) => find(permission));
    };
    return (react_1.default.createElement(AuthContext.Provider, { value: {
            user,
            logIn,
            loading,
            logOut,
            forgot,
            forgotVerify,
            init,
            updatePassword,
            filterPermissions,
            hasPermission,
            authStatus,
            setAuthStatus
        } }, children));
};
exports.AuthContextProvider = AuthContextProvider;
const useAuth = () => (0, react_1.useContext)(AuthContext);
exports.useAuth = useAuth;
