"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const react_2 = require("@fluentui/react");
const react_beautiful_dnd_1 = require("react-beautiful-dnd");
const fields_select_panel_1 = __importDefault(require("./fields-select-panel"));
const content_types_context_1 = require("../context/content-types.context");
const fields_configure_panel_1 = __importDefault(require("./fields-configure-panel"));
const theme = (0, react_2.getTheme)();
const styles = (0, react_2.mergeStyleSets)({
    fields: {
        marginBottom: 16,
    },
    field: {
        display: 'flex',
        minHeight: 0,
    },
    fieldGroup: {
        padding: '8px 30px',
    },
    fieldWrapper: {
        padding: 10,
        border: `1px solid ${theme.palette.neutralQuaternary}`,
        borderRadius: theme.effects.roundedCorner2,
        marginBottom: 10,
    },
    fieldWrapperLight: {
        backgroundColor: theme.palette.neutralLighterAlt,
    },
    fieldWrapperWhite: {
        backgroundColor: theme.palette.white,
    },
    icon: {
        height: 30,
        width: 30,
        fontSize: 16,
        backgroundColor: theme.palette.neutralLighter,
        display: 'flex',
        justifyContent: 'center',
        alignItems: 'center',
    },
    iconDrag: {
        width: 30,
    },
    content: {
        marginLeft: 10,
        marginRight: 10,
        flexGrow: 1,
    },
    fieldName: {
        fontSize: 16,
        fontWeight: 600,
        whiteSpace: 'nowrap',
        overflow: 'hidden',
        textOverflow: 'ellipsis',
    },
    fieldHelper: {
        fontSize: 12,
        color: theme.palette.neutralTertiary,
        marginBottom: 10,
    },
    fieldContent: {
        padding: '8px 40px',
    },
    fieldContentItem: {
        fontSize: 12,
        padding: `10px 0`,
        '&:not(:last-child)': {
            borderBottom: `1px solid ${theme.palette.neutralLight}`,
        },
    },
    actions: {
        margin: '24px 0 12px',
    },
});
const reorder = (list, startIndex, endIndex) => {
    const result = Array.from(list);
    const [removed] = result.splice(startIndex, 1);
    result.splice(endIndex, 0, removed);
    return result;
};
const FieldsList = ({ field, isRoot }) => {
    const [error, setError] = (0, react_1.useState)(null);
    const [editingField, setEditingField] = (0, react_1.useState)(null);
    const [wa, setWa] = (0, react_1.useState)(false);
    const { getFields, getComponents, getContentTypes } = (0, content_types_context_1.useContentTypes)();
    if (!(field === null || field === void 0 ? void 0 : field.fields)) {
        field.fields = [];
    }
    const getFieldFn = (0, react_1.useCallback)((item) => {
        var _a;
        return ((_a = getFields === null || getFields === void 0 ? void 0 : getFields.result) !== null && _a !== void 0 ? _a : []).find((field) => field.type === item.type);
    }, [getFields === null || getFields === void 0 ? void 0 : getFields.result]);
    const getComponentFn = (0, react_1.useCallback)((name) => {
        var _a;
        return ((_a = getComponents === null || getComponents === void 0 ? void 0 : getComponents.result) !== null && _a !== void 0 ? _a : []).find((component) => component.name == name);
    }, [getComponents === null || getComponents === void 0 ? void 0 : getComponents.result]);
    const getPostFn = (0, react_1.useCallback)((name) => {
        var _a;
        return ((_a = getContentTypes === null || getContentTypes === void 0 ? void 0 : getContentTypes.result) !== null && _a !== void 0 ? _a : []).find((post) => post.name == name);
    }, [getContentTypes === null || getContentTypes === void 0 ? void 0 : getContentTypes.result]);
    (0, react_1.useEffect)(() => {
        getFields.execute();
        if (!(getComponents === null || getComponents === void 0 ? void 0 : getComponents.result) && !(getComponents === null || getComponents === void 0 ? void 0 : getComponents.loading)) {
            getComponents.execute();
        }
        if (!(getContentTypes === null || getContentTypes === void 0 ? void 0 : getContentTypes.result) && !(getContentTypes === null || getContentTypes === void 0 ? void 0 : getContentTypes.loading)) {
            getContentTypes.execute({ type: 'page,post' });
        }
    }, []);
    const [selectFieldOpen, setSelectFieldOpen] = (0, react_1.useState)(false);
    const removeAt = (index) => {
        field.fields.splice(index, 1);
        setWa(!wa);
    };
    const getItemStyle = (isDragging, draggableStyle) => {
        if (isDragging) {
            return Object.assign({ userSelect: 'none', boxShadow: `${theme.effects.elevation64} !important` }, draggableStyle);
        }
        return Object.assign({}, draggableStyle);
    };
    const renderZoneComponents = (items = '') => {
        const components = items.split(',').filter((item) => (item === null || item === void 0 ? void 0 : item.length) > 0);
        return (react_1.default.createElement("div", { className: styles.fieldContent },
            react_1.default.createElement(react_2.Label, null, "Allowed components"),
            components.map(getComponentFn).map((component) => (react_1.default.createElement("div", { key: component === null || component === void 0 ? void 0 : component.name, className: styles.fieldContentItem }, component === null || component === void 0 ? void 0 : component.name))),
            (components === null || components === void 0 ? void 0 : components.length) === 0 && (react_1.default.createElement(react_2.Separator, null,
                react_1.default.createElement("span", { style: { fontSize: 12 } }, "No components")))));
    };
    const renderRelationPosts = (ids = '') => {
        const posts = ids.split(',').filter((item) => (item === null || item === void 0 ? void 0 : item.length) > 0);
        return (react_1.default.createElement("div", { className: styles.fieldContent },
            react_1.default.createElement(react_2.Label, null, "Allowed relation posts"),
            posts.map(getPostFn).map((post) => (react_1.default.createElement("div", { key: post === null || post === void 0 ? void 0 : post.name, className: styles.fieldContentItem }, post === null || post === void 0 ? void 0 : post.name))),
            (posts === null || posts === void 0 ? void 0 : posts.length) === 0 && (react_1.default.createElement(react_2.Separator, null,
                react_1.default.createElement("span", { style: { fontSize: 12 } }, "No posts")))));
    };
    const onDragEnd = (result) => {
        // dropped outside the list
        if (!result.destination) {
            return;
        }
        const items = reorder(field.fields, result.source.index, result.destination.index);
        field.fields = items;
    };
    const move = (index, offset) => {
        const items = reorder(field.fields, index, index + offset);
        field.fields = items;
        setWa(!wa);
    };
    return (react_1.default.createElement(react_1.default.Fragment, null,
        react_1.default.createElement("div", null,
            react_1.default.createElement(react_beautiful_dnd_1.DragDropContext, { onDragEnd: onDragEnd },
                react_1.default.createElement(react_beautiful_dnd_1.Droppable, { droppableId: "droppable" }, (provided) => {
                    var _a;
                    return (react_1.default.createElement("div", Object.assign({}, provided.droppableProps, { className: styles.fields, ref: provided.innerRef }),
                        ((_a = field === null || field === void 0 ? void 0 : field.fields) !== null && _a !== void 0 ? _a : []).map((item, index) => (react_1.default.createElement(react_beautiful_dnd_1.Draggable, { key: item === null || item === void 0 ? void 0 : item.name, draggableId: item === null || item === void 0 ? void 0 : item.name, index: index }, (provided, snapshot) => {
                            var _a, _b, _c, _d, _e, _f;
                            return (react_1.default.createElement("div", Object.assign({ ref: provided.innerRef, className: `${styles.fieldWrapper} ${item.type === 'tab'
                                    ? styles.fieldWrapperLight
                                    : styles.fieldWrapperWhite}` }, provided.draggableProps, provided.dragHandleProps, { style: getItemStyle(snapshot.isDragging, provided.draggableProps.style), "data-cy": `contentTypes-fieldsList-item-${item === null || item === void 0 ? void 0 : item.name}` }),
                                react_1.default.createElement("div", { className: styles.field },
                                    react_1.default.createElement("div", { className: styles.icon },
                                        react_1.default.createElement(react_2.Icon, Object.assign({}, ((_b = (_a = getFieldFn(item)) === null || _a === void 0 ? void 0 : _a.iconProps) !== null && _b !== void 0 ? _b : {})))),
                                    react_1.default.createElement("div", { className: styles.content },
                                        react_1.default.createElement("div", { className: styles.fieldName }, ((_c = item === null || item === void 0 ? void 0 : item.label) === null || _c === void 0 ? void 0 : _c.length) ? item === null || item === void 0 ? void 0 : item.label : item === null || item === void 0 ? void 0 : item.name),
                                        react_1.default.createElement("div", { className: styles.fieldHelper }, item === null || item === void 0 ? void 0 : item.type)),
                                    react_1.default.createElement("div", null,
                                        react_1.default.createElement(react_2.Stack, { horizontal: true, tokens: { childrenGap: 8 } },
                                            react_1.default.createElement(react_2.IconButton, { disabled: index === 0, onClick: () => move(index, -1), iconProps: { iconName: 'Up' }, title: "Move Up", ariaLabel: "Move Up" }),
                                            react_1.default.createElement(react_2.IconButton, { disabled: index >= ((_d = field === null || field === void 0 ? void 0 : field.fields) === null || _d === void 0 ? void 0 : _d.length) - 1, onClick: () => move(index, 1), iconProps: { iconName: 'Down' }, title: "Move Down", ariaLabel: "Move Down" }),
                                            react_1.default.createElement(react_2.IconButton, { onClick: () => setEditingField(Object.assign(Object.assign({}, item), { index })), iconProps: { iconName: 'Edit' }, title: "Edit", ariaLabel: "Edit", selected: selectFieldOpen }),
                                            react_1.default.createElement(react_2.IconButton, { onClick: () => removeAt(index), iconProps: { iconName: 'Delete' }, title: "Delete", ariaLabel: "Delete" })))),
                                ((item === null || item === void 0 ? void 0 : item.type) === 'group' ||
                                    (item === null || item === void 0 ? void 0 : item.type) === 'repeatable') && (react_1.default.createElement("div", { className: styles.fieldGroup },
                                    react_1.default.createElement(FieldsList, { field: item }))),
                                (item === null || item === void 0 ? void 0 : item.type) === 'zone' &&
                                    renderZoneComponents(item === null || item === void 0 ? void 0 : item.components),
                                (item === null || item === void 0 ? void 0 : item.type) === 'custom' && (react_1.default.createElement("div", { className: styles.fieldContent }, ((_e = item.component) === null || _e === void 0 ? void 0 : _e.length) > 0 && (react_1.default.createElement(react_1.default.Fragment, null,
                                    react_1.default.createElement(react_2.Label, null, "Component"),
                                    react_1.default.createElement("div", { className: styles.fieldContentItem }, (_f = getComponentFn(item.component)) === null || _f === void 0 ? void 0 : _f.name))))),
                                (item === null || item === void 0 ? void 0 : item.type) === 'relation' &&
                                    renderRelationPosts(item === null || item === void 0 ? void 0 : item.posts)));
                        }))),
                        provided.placeholder));
                })),
            react_1.default.createElement("div", { className: styles.actions },
                react_1.default.createElement(react_2.Separator, null,
                    react_1.default.createElement(react_2.ActionButton, { iconProps: {
                            iconName: 'Add',
                        }, onClick: () => setSelectFieldOpen(true), "data-cy": "contentTypes-fieldsList-addField" }, isRoot ? 'Add field' : 'Add sub-field')))),
        react_1.default.createElement(fields_select_panel_1.default, { isOpen: selectFieldOpen, onDismiss: () => setSelectFieldOpen(false), error: error, onSubmit: (e) => {
                var _a;
                setError(null);
                if (((_a = field === null || field === void 0 ? void 0 : field.fields) !== null && _a !== void 0 ? _a : []).find((field) => (field === null || field === void 0 ? void 0 : field.name) === (e === null || e === void 0 ? void 0 : e.name))) {
                    setError('duplicate_field_name');
                }
                else {
                    field.fields.push(e);
                    setSelectFieldOpen(false);
                }
            } }),
        react_1.default.createElement(fields_configure_panel_1.default, { isOpen: !!editingField, type: editingField === null || editingField === void 0 ? void 0 : editingField.type, field: editingField, onDismiss: () => setEditingField(null), onSubmit: (e) => {
                var _a;
                setError(null);
                if (((_a = field === null || field === void 0 ? void 0 : field.fields) !== null && _a !== void 0 ? _a : []).find((field, index) => e.index !== index && (field === null || field === void 0 ? void 0 : field.name) === (e === null || e === void 0 ? void 0 : e.name))) {
                    setError('duplicate_field_name');
                }
                else {
                    const { index } = e;
                    const tmpField = Object.assign({}, field === null || field === void 0 ? void 0 : field.fields[index]);
                    delete e.index;
                    field.fields[index] = Object.assign(Object.assign({}, tmpField), e);
                    setEditingField(null);
                }
            }, error: error })));
};
exports.default = FieldsList;
