"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.exportPosts = exports.importPosts = exports.importPost = exports.getParent = void 0;
const typeorm_1 = require("typeorm");
const async_1 = __importDefault(require("async"));
const orm_helpers_1 = require("../common/orm-helpers");
const post_model_1 = __importDefault(require("../models/post.model"));
const post_controller_1 = require("../controllers/post.controller");
const content_type_model_1 = __importDefault(require("../models/content-type.model"));
const logger_1 = __importDefault(require("../../shared/features/logger"));
const tags_bl_1 = require("./tags.bl");
const mappers_1 = require("../common/mappers");
const POST_FOLDER_TYPE = 'folder';
const POST_HIERARCHICAL_TYPE = 'hierarchical_post';
const getParent = (manager, components, parent) => __awaiter(void 0, void 0, void 0, function* () {
    if (components.length > 0) {
        let newParent;
        const slug = components.shift();
        const slugPath = parent ? `${parent.slugPath}/${slug}` : slug;
        try {
            newParent = yield manager.findOne(post_model_1.default, {
                slugPath,
            });
            if (!newParent) {
                newParent = yield manager.save(post_model_1.default, {
                    parent,
                    name: slug,
                    type: POST_FOLDER_TYPE,
                    slug,
                    slugPath,
                });
            }
        }
        catch (err) {
            newParent = yield manager.findOne(post_model_1.default, {
                slugPath,
            });
        }
        parent = yield (0, exports.getParent)(manager, components, newParent);
    }
    return parent;
});
exports.getParent = getParent;
const importPost = ({ manager, post, user, options }) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    let saved;
    const searchObj = {
        slugPath: post.slugPath,
    };
    logger_1.default.info(`Importing ${post.slugPath}`);
    saved = yield manager.findOne(post_model_1.default, {
        relations: [
            'meta',
            'contentType',
            'author',
            'author.meta',
            'parent',
            'tags',
        ],
        where: searchObj,
    });
    if (saved && !(options === null || options === void 0 ? void 0 : options.force)) {
        logger_1.default.info(`Skipping ${post.slugPath}, exists.`);
        return saved;
    }
    if ((saved === null || saved === void 0 ? void 0 : saved.type) === POST_FOLDER_TYPE ||
        (saved &&
            (((_a = saved === null || saved === void 0 ? void 0 : saved.contentType) === null || _a === void 0 ? void 0 : _a.name) !== ((_b = post.contentType) === null || _b === void 0 ? void 0 : _b.name) ||
                (saved === null || saved === void 0 ? void 0 : saved.type) !== (post === null || post === void 0 ? void 0 : post.type)))) {
        logger_1.default.info(`Skipping ${post.slugPath}, existing post either of type "folder" or not matching contentType.`);
        return saved;
    }
    let contentType;
    let postContentType;
    const metaPostContentType = (_c = ((post === null || post === void 0 ? void 0 : post.meta) || []).find((meta) => meta.key === 'postContentType')) === null || _c === void 0 ? void 0 : _c.value;
    if (post.type === POST_HIERARCHICAL_TYPE && metaPostContentType) {
        postContentType = yield manager.findOne(content_type_model_1.default, {
            where: {
                name: metaPostContentType === null || metaPostContentType === void 0 ? void 0 : metaPostContentType.name,
            },
        });
        if (!postContentType) {
            postContentType = yield manager.save(content_type_model_1.default, {
                name: metaPostContentType === null || metaPostContentType === void 0 ? void 0 : metaPostContentType.name,
                type: metaPostContentType === null || metaPostContentType === void 0 ? void 0 : metaPostContentType.type,
                author: user,
                fields: JSON.stringify((metaPostContentType === null || metaPostContentType === void 0 ? void 0 : metaPostContentType.fields) || []),
            });
        }
    }
    else if (post.type === POST_HIERARCHICAL_TYPE) {
        logger_1.default.info(`Skipping ${post.slugPath}, hierarchical post doesn't have valid postContentTypeId.`);
        return saved;
    }
    let tags;
    if (((_d = post === null || post === void 0 ? void 0 : post.tags) === null || _d === void 0 ? void 0 : _d.length) > 0) {
        tags = yield Promise.all(post === null || post === void 0 ? void 0 : post.tags.map((tag) => __awaiter(void 0, void 0, void 0, function* () {
            const imported = yield (0, tags_bl_1.importTag)({
                manager,
                tag,
                user,
            });
            return imported;
        })));
    }
    const newMeta = (post.meta || [])
        .map((item) => ({
        key: item.key,
        value: item.value,
    }))
        .filter((item) => item.key !== 'postContentType');
    if (postContentType) {
        newMeta.push({ key: 'postContentTypeId', value: postContentType.id });
    }
    if (saved) {
        yield (0, post_controller_1.createPostVersion)(manager.getRepository(post_model_1.default), saved, user);
        yield (0, orm_helpers_1.updateMeta)(manager, post_model_1.default, saved, newMeta);
        if ((tags === null || tags === void 0 ? void 0 : tags.length) > 0) {
            saved.tags = tags;
            yield manager.save(post_model_1.default, saved);
        }
        logger_1.default.info(`Updating existing post ${post.slugPath} success.`);
        return saved;
    }
    if ((post === null || post === void 0 ? void 0 : post.type) !== POST_FOLDER_TYPE) {
        contentType = yield manager.findOne(content_type_model_1.default, {
            where: {
                name: (_e = post === null || post === void 0 ? void 0 : post.contentType) === null || _e === void 0 ? void 0 : _e.name,
            },
        });
    }
    if (!contentType && (post === null || post === void 0 ? void 0 : post.type) !== POST_FOLDER_TYPE) {
        contentType = yield manager.save(content_type_model_1.default, {
            name: (_f = post === null || post === void 0 ? void 0 : post.contentType) === null || _f === void 0 ? void 0 : _f.name,
            type: (_g = post === null || post === void 0 ? void 0 : post.contentType) === null || _g === void 0 ? void 0 : _g.type,
            author: user,
            fields: JSON.stringify(((_h = post === null || post === void 0 ? void 0 : post.contentType) === null || _h === void 0 ? void 0 : _h.fields) || []),
        });
    }
    let parent;
    const slugComponents = post.slugPath
        .split('/')
        .filter((cmp) => (cmp === null || cmp === void 0 ? void 0 : cmp.length) > 0);
    slugComponents.pop();
    if (slugComponents.length > 0) {
        parent = yield (0, exports.getParent)(manager, slugComponents);
    }
    const postObj = Object.assign({ name: post.name, slug: post.slug, slugPath: post.slugPath, type: post.type, contentType,
        parent,
        tags, author: user, meta: newMeta, status: (options === null || options === void 0 ? void 0 : options.publish) ? 'published' : 'draft' }, ((options === null || options === void 0 ? void 0 : options.publish) ? {
        publishedAt: new Date()
    } : {}));
    saved = yield manager.save(post_model_1.default, postObj);
    logger_1.default.info(`Created new post ${post.slugPath}.`);
    return saved;
});
exports.importPost = importPost;
const importPosts = ({ entityManager, data, user, options }) => __awaiter(void 0, void 0, void 0, function* () {
    const sorted = (data || []).sort((a, b) => {
        if (a.slugPath < b.slugPath) {
            return -1;
        }
        if (a.slugPath > b.slugPath) {
            return 1;
        }
        return 0;
    });
    if ((sorted === null || sorted === void 0 ? void 0 : sorted.length) > 0) {
        logger_1.default.info('Importing posts.');
    }
    yield async_1.default.eachLimit(sorted, 1, (item, next) => __awaiter(void 0, void 0, void 0, function* () {
        try {
            yield (0, exports.importPost)({
                manager: entityManager,
                user,
                post: item,
                options
            });
            next();
        }
        catch (e) {
            next(e);
        }
    }));
    if ((sorted === null || sorted === void 0 ? void 0 : sorted.length) > 0) {
        logger_1.default.info('Importing posts successful.');
    }
});
exports.importPosts = importPosts;
const exportPosts = ({ entityManager, }) => __awaiter(void 0, void 0, void 0, function* () {
    const postRepository = entityManager.getRepository(post_model_1.default);
    const contentTypeRepository = entityManager.getRepository(content_type_model_1.default);
    let posts = yield postRepository.find({
        relations: ['contentType', 'tags', 'meta'],
        where: {
            type: (0, typeorm_1.In)(['post', 'page', 'hierarchical_post', 'fragment', 'folder']),
        },
    });
    posts = yield Promise.all(posts.map((post) => __awaiter(void 0, void 0, void 0, function* () {
        var _j;
        if (post.type === POST_HIERARCHICAL_TYPE) {
            const contentTypeId = (_j = (post.meta || []).find((meta) => (meta === null || meta === void 0 ? void 0 : meta.key) === 'postContentTypeId')) === null || _j === void 0 ? void 0 : _j.value;
            if (contentTypeId !== undefined) {
                const contentType = yield contentTypeRepository.findOne(contentTypeId);
                post.meta = post.meta.filter((meta) => meta.key !== 'postContentTypeId');
                post.meta.push({
                    key: 'postContentType',
                    value: (0, mappers_1.mapContentType)(contentType),
                });
            }
        }
        return post;
    })));
    return posts.map(mappers_1.mapPost);
});
exports.exportPosts = exportPosts;
