"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = __importDefault(require("express"));
const async_middleware_1 = __importDefault(require("../middleware/async.middleware"));
const typeorm_1 = require("typeorm");
const group_model_1 = __importDefault(require("../models/group.model"));
const lodash_1 = __importDefault(require("lodash"));
const orm_helpers_1 = require("../common/orm-helpers");
const yup = __importStar(require("yup"));
const not_found_error_1 = __importDefault(require("../errors/not-found-error"));
const auth_middleware_1 = __importDefault(require("../middleware/auth.middleware"));
const app = (0, express_1.default)();
app.get('/groups', (0, auth_middleware_1.default)(['users_administration']), (0, async_middleware_1.default)((req, res) => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    const groups = yield group_model_1.default.find({
        relations: (0, orm_helpers_1.createRelationsArray)((_a = req.query) === null || _a === void 0 ? void 0 : _a.expand),
    });
    res.send(groups);
})));
app.get('/groups/:id', (0, auth_middleware_1.default)(['users_administration']), (0, async_middleware_1.default)((req, res) => __awaiter(void 0, void 0, void 0, function* () {
    var _b;
    const group = yield group_model_1.default.findOne({
        relations: (0, orm_helpers_1.createRelationsArray)((_b = req.query) === null || _b === void 0 ? void 0 : _b.expand),
        where: { id: req.params.id },
    });
    res.send(group);
})));
app.post('/groups', (0, auth_middleware_1.default)(['users_administration']), (0, async_middleware_1.default)((req, res) => __awaiter(void 0, void 0, void 0, function* () {
    var _c, _d;
    const group = lodash_1.default.pick(req.body, ['name', 'permissions', 'description']);
    yield req.validate({
        name: yup
            .string()
            .required()
            .test('name-exists', 'Name exists.', () => __awaiter(void 0, void 0, void 0, function* () {
            const existingGroup = yield group_model_1.default.findOne({ name: group.name });
            return existingGroup === undefined;
        })),
        permissions: yup.array(),
        description: yup.string(),
        userIds: yup.array(),
    });
    const users = (_d = (_c = req.body) === null || _c === void 0 ? void 0 : _c.users) !== null && _d !== void 0 ? _d : [];
    const userIds = users.map((u) => u.id);
    yield (0, typeorm_1.getManager)().transaction((entityManager) => __awaiter(void 0, void 0, void 0, function* () {
        const groupRepository = (0, orm_helpers_1.getEnhancedRepository)(group_model_1.default, entityManager);
        const record = yield groupRepository.save(group);
        if (Array.isArray(userIds)) {
            yield groupRepository.attach(record.id, 'users', userIds);
        }
        res.send(record);
    }));
})));
app.put('/groups/:id', (0, auth_middleware_1.default)(['users_administration']), (0, async_middleware_1.default)((req, res) => __awaiter(void 0, void 0, void 0, function* () {
    var _e, _f, _g;
    const id = (_e = req.params) === null || _e === void 0 ? void 0 : _e.id;
    const group = lodash_1.default.pick(req.body, ['name', 'permissions', 'description']);
    const groupModel = yield group_model_1.default.findOne({ id });
    if (!groupModel)
        throw new not_found_error_1.default('not_found');
    yield req.validate({
        name: yup
            .string()
            .required()
            .test('name-exists', 'Name exists.', () => __awaiter(void 0, void 0, void 0, function* () {
            const existingGroup = yield group_model_1.default.findOne({ name: group.name });
            return existingGroup === undefined || existingGroup.id === Number(id);
        })),
        permissions: yup.array(),
        description: yup.string(),
        userIds: yup.array(),
    });
    const users = (_g = (_f = req.body) === null || _f === void 0 ? void 0 : _f.users) !== null && _g !== void 0 ? _g : [];
    const userIds = users.map((u) => u.id);
    // Protect auth from removing themselves from the protected group
    if (Array.isArray(userIds) &&
        groupModel.protected &&
        lodash_1.default.map(groupModel.users, 'id').includes(req.data.user.id) &&
        !userIds.includes(req.data.user.id)) {
        userIds.push(req.data.user.id);
    }
    yield (0, typeorm_1.getManager)().transaction((entityManager) => __awaiter(void 0, void 0, void 0, function* () {
        const groupRepository = (0, orm_helpers_1.getEnhancedRepository)(group_model_1.default, entityManager);
        yield groupRepository.update({ id, protected: false }, group);
        if (Array.isArray(userIds)) {
            yield groupRepository.sync(id, 'users', userIds);
        }
        const groupResponse = yield groupRepository.findOne({ where: { id } });
        res.send(groupResponse);
    }));
})));
app.delete('/groups', (0, auth_middleware_1.default)(['users_administration']), (0, async_middleware_1.default)((req, res) => __awaiter(void 0, void 0, void 0, function* () {
    var _h, _j;
    const groupRepository = (0, orm_helpers_1.getEnhancedRepository)(group_model_1.default);
    yield groupRepository.delete({
        id: (0, typeorm_1.In)((_j = (_h = req.body) === null || _h === void 0 ? void 0 : _h.ids) !== null && _j !== void 0 ? _j : []),
        protected: false,
    });
    res.send();
})));
app.delete('/groups/:id', (0, auth_middleware_1.default)(['users_administration']), (0, async_middleware_1.default)((req, res) => __awaiter(void 0, void 0, void 0, function* () {
    var _k;
    const groupRepository = (0, orm_helpers_1.getEnhancedRepository)(group_model_1.default);
    const group = yield groupRepository.findOne({
        id: (_k = req.params) === null || _k === void 0 ? void 0 : _k.id,
        protected: false,
    });
    if (!group)
        throw new not_found_error_1.default('not_found');
    yield group.remove();
    res.send();
})));
exports.default = app;
