"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = __importDefault(require("express"));
const auth_middleware_1 = __importDefault(require("../middleware/auth.middleware"));
const async_middleware_1 = __importDefault(require("../middleware/async.middleware"));
const typeorm_1 = require("typeorm");
const tag_model_1 = __importDefault(require("../models/tag.model"));
const yup = __importStar(require("yup"));
const bad_request_error_1 = __importDefault(require("../errors/bad-request-error"));
const orm_helpers_1 = require("../common/orm-helpers");
const mappers_1 = require("../common/mappers");
const content_middleware_1 = __importDefault(require("../middleware/content.middleware"));
const utility_1 = require("../../admin/helpers/utility");
const app = (0, express_1.default)();
app.get('/tags', (0, auth_middleware_1.default)(), (0, async_middleware_1.default)((req, res) => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    const search = (_a = req === null || req === void 0 ? void 0 : req.query) === null || _a === void 0 ? void 0 : _a.search;
    const tagRepository = (0, typeorm_1.getRepository)(tag_model_1.default);
    const tagsQb = tagRepository
        .createQueryBuilder('tag')
        .leftJoinAndSelect('tag.meta', 'meta')
        .leftJoinAndSelect('tag.author', 'author')
        .leftJoinAndSelect('author.meta', 'author.meta');
    if ((search === null || search === void 0 ? void 0 : search.length) > 0) {
        const searchWhere = new typeorm_1.Brackets((qb) => {
            qb.where('LOWER(tag.name) LIKE :search', {
                search: `%${search.toLowerCase()}%`,
            }).orWhere('LOWER(tag.slug) LIKE :search', {
                search: `%${search.toLowerCase()}%`,
            });
        });
        tagsQb.where(searchWhere);
    }
    tagsQb.orderBy('tag.name', 'DESC');
    const tags = yield tagsQb.getMany();
    res.send(tags.map(mappers_1.mapTag));
})));
app.post('/tags', (0, auth_middleware_1.default)(['tags_create']), (0, async_middleware_1.default)((req, res) => __awaiter(void 0, void 0, void 0, function* () {
    yield req.validate({
        name: yup.string().max(256).required(),
        slug: yup.string().max(256).required(),
    }, 'body');
    const entityManager = (0, typeorm_1.getManager)();
    const params = req.body;
    let tag;
    try {
        yield entityManager.transaction((transactionManager) => __awaiter(void 0, void 0, void 0, function* () {
            var _b;
            let parent;
            if (params.parentId) {
                parent = yield transactionManager.findOne(tag_model_1.default, {
                    id: params.parentId,
                });
                if (!parent)
                    throw new bad_request_error_1.default('invalid_parent');
            }
            const tagObj = {
                name: params.name,
                slug: params.slug,
                author: (_b = req === null || req === void 0 ? void 0 : req.data) === null || _b === void 0 ? void 0 : _b.user,
            };
            if (parent) {
                tagObj.parent = parent;
                tagObj.slugPath = `${parent.slugPath}/${params.slug}`;
            }
            else {
                tagObj.slugPath = params.slug;
            }
            tag = yield transactionManager.save(tag_model_1.default, tagObj);
        }));
    }
    catch (err) {
        if ((err === null || err === void 0 ? void 0 : err.code) === '23505') {
            throw new bad_request_error_1.default('duplicate_slug');
        }
        throw err;
    }
    return res.send((0, mappers_1.mapTag)(tag));
})));
app.delete('/tags', (0, auth_middleware_1.default)(['tags_delete']), (0, async_middleware_1.default)((req, res) => __awaiter(void 0, void 0, void 0, function* () {
    var _c;
    const ids = (_c = req === null || req === void 0 ? void 0 : req.body) !== null && _c !== void 0 ? _c : [];
    if (!ids || (ids === null || ids === void 0 ? void 0 : ids.length) === 0)
        return res.send([]);
    const tagRepository = (0, typeorm_1.getRepository)(tag_model_1.default);
    const deleted = yield tagRepository.delete({
        id: (0, typeorm_1.In)(ids),
    });
    return res.send(deleted);
})));
app.put('/tags/:tagId', (0, auth_middleware_1.default)(['tags_update']), (0, async_middleware_1.default)((req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const entityManager = (0, typeorm_1.getManager)();
    let tag;
    try {
        yield entityManager.transaction((transactionManager) => __awaiter(void 0, void 0, void 0, function* () {
            var _d;
            const tagRepository = transactionManager.getRepository(tag_model_1.default);
            tag = yield tagRepository.findOne({
                relations: ['meta', 'parent', 'author', 'author.meta'],
                where: {
                    id: req.params.tagId,
                },
            });
            if (!tag)
                throw new bad_request_error_1.default('invalid_tag');
            if (req.body.slug) {
                const newSlug = req.body.slug;
                const oldSlugPath = tag === null || tag === void 0 ? void 0 : tag.slugPath;
                const newSlugPath = (tag === null || tag === void 0 ? void 0 : tag.parent)
                    ? `${(_d = tag === null || tag === void 0 ? void 0 : tag.parent) === null || _d === void 0 ? void 0 : _d.slugPath}/${req.body.slug}`
                    : req.body.slug;
                tag.slug = newSlug;
                tag.slugPath = newSlugPath;
                yield transactionManager.query((0, orm_helpers_1.getReplaceChildrenQuery)('tag', 'slugPath', oldSlugPath, newSlugPath));
            }
            if (req.body.name) {
                tag.name = req.body.name;
            }
            tag.updatedAt = new Date();
            yield tagRepository.save(tag);
        }));
    }
    catch (err) {
        if ((err === null || err === void 0 ? void 0 : err.code) === '23505') {
            throw new bad_request_error_1.default('duplicate_slug');
        }
        throw err;
    }
    res.send((0, mappers_1.mapTag)(tag));
})));
app.get('/search/tags', (0, content_middleware_1.default)({ alwaysAuthorize: true }), (0, async_middleware_1.default)((req, res) => __awaiter(void 0, void 0, void 0, function* () {
    var _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s;
    const tagRepository = (0, orm_helpers_1.getEnhancedRepository)(tag_model_1.default);
    const qb = tagRepository.createQueryBuilder('tag');
    const search = (_e = req === null || req === void 0 ? void 0 : req.query) === null || _e === void 0 ? void 0 : _e.search;
    const parent = (_f = req === null || req === void 0 ? void 0 : req.query) === null || _f === void 0 ? void 0 : _f.parent;
    const onlyOrphans = (_g = req === null || req === void 0 ? void 0 : req.query) === null || _g === void 0 ? void 0 : _g.onlyOrphans;
    const slugPath = (_h = req === null || req === void 0 ? void 0 : req.query) === null || _h === void 0 ? void 0 : _h.slugPath;
    let limit = 100;
    if (((_j = req === null || req === void 0 ? void 0 : req.query) === null || _j === void 0 ? void 0 : _j.limit) && !Number.isNaN(Number((_k = req === null || req === void 0 ? void 0 : req.query) === null || _k === void 0 ? void 0 : _k.limit))) {
        limit = Number((_l = req === null || req === void 0 ? void 0 : req.query) === null || _l === void 0 ? void 0 : _l.limit);
    }
    let page = 1;
    if (((_m = req === null || req === void 0 ? void 0 : req.query) === null || _m === void 0 ? void 0 : _m.page) && !Number.isNaN(Number((_o = req === null || req === void 0 ? void 0 : req.query) === null || _o === void 0 ? void 0 : _o.page))) {
        page = Number((_p = req === null || req === void 0 ? void 0 : req.query) === null || _p === void 0 ? void 0 : _p.page);
    }
    const orderBy = (_q = req === null || req === void 0 ? void 0 : req.query) === null || _q === void 0 ? void 0 : _q.orderBy;
    const order = (_r = req === null || req === void 0 ? void 0 : req.query) === null || _r === void 0 ? void 0 : _r.order;
    const expand = (((_s = req === null || req === void 0 ? void 0 : req.query) === null || _s === void 0 ? void 0 : _s.expand) || '').split(',');
    if (parent || expand.find((val) => val === 'parent')) {
        qb.leftJoinAndSelect('tag.parent', 'parent');
    }
    if (slugPath) {
        qb.andWhere('tag.slugPath IN (:...slugPaths)', {
            slugPaths: slugPath.split(','),
        });
    }
    if ((0, utility_1.isTrue)(onlyOrphans)) {
        if (parent) {
            qb.andWhere('parent.slugPath = :parentSlugPath', {
                parentSlugPath: parent,
            });
        }
        else {
            qb.andWhere('tag.parentId IS NULL');
        }
    }
    else if (parent) {
        qb.andWhere('tag.slugPath LIKE :parentSlugPath', {
            parentSlugPath: `${parent.toLowerCase()}/%`,
        });
    }
    if ((search === null || search === void 0 ? void 0 : search.length) > 0) {
        qb.andWhere(new typeorm_1.Brackets((subQb) => {
            subQb
                .where('LOWER(tag.name) LIKE :search', {
                search: `%${search.toLowerCase()}%`,
            })
                .orWhere('LOWER(tag.slugPath) LIKE :search', {
                search: `%${search.toLowerCase()}%`,
            });
        }));
    }
    const getMany = () => {
        qb.take(limit).skip((page - 1) * limit);
        return qb.getMany();
    };
    qb.addOrderBy(orderBy || 'tag.slugPath', order === 'DESC' ? 'DESC' : 'ASC');
    const [count, results] = yield Promise.all([
        qb.getCount(),
        getMany()
    ]);
    res.send({
        count,
        results: (results || []).map((tag) => (0, mappers_1.mapPublicTag)(tag)),
        page,
        limit
    });
})));
exports.default = app;
