"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.connectDatabaseDriver = exports.databaseDriver = void 0;
const chalk_1 = __importDefault(require("chalk"));
const typeorm_1 = require("typeorm");
const hooks_1 = __importDefault(require("../../shared/features/hooks"));
const sqlite_patch_1 = require("./sqlite-patch");
const database_util_1 = require("../../scripts/util/database.util");
const databaseDriver = {
    connection: undefined,
};
exports.databaseDriver = databaseDriver;
const connectDatabaseDriver = () => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const entities = yield hooks_1.default.applyFilters('db/models', []);
        const databaseType = (0, database_util_1.getDatabaseType)(process.env.TYPEORM_CONNECTION);
        const defaultConnectionOptions = {
            entities,
            type: databaseType,
        };
        (0, sqlite_patch_1.sqlitePatch)();
        const connectionOptions = yield hooks_1.default.applyFilters('db/options', defaultConnectionOptions);
        const typeormConnectionOptions = yield (0, typeorm_1.getConnectionOptions)();
        const connection = yield (0, typeorm_1.createConnection)(Object.assign(Object.assign({}, typeormConnectionOptions), connectionOptions));
        databaseDriver.connection = yield hooks_1.default.applyFilters('db/connection', connection);
        yield hooks_1.default.doAction('db/init', connection, connectionOptions);
        console.log(chalk_1.default.green('Connection to database established.'));
    }
    catch (e) {
        console.log(e);
        console.error(chalk_1.default.red('Error establishing connection to database.'));
        process.exit(1);
    }
});
exports.connectDatabaseDriver = connectDatabaseDriver;
