"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sendMail = exports.mailDriver = exports.connectMailDriver = void 0;
const nodemailer_1 = __importDefault(require("nodemailer"));
const chalk_1 = __importDefault(require("chalk"));
const hooks_1 = __importDefault(require("../../shared/features/hooks"));
const logger_1 = __importDefault(require("../../shared/features/logger"));
const mailDriver = {
    transporter: undefined
};
exports.mailDriver = mailDriver;
const connectMailDriver = () => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const transport = yield hooks_1.default.applyFilters('mail/options', null);
        if (!transport) {
            return;
        }
        mailDriver.transporter = nodemailer_1.default.createTransport(transport);
        yield hooks_1.default.doAction('mail/init', mailDriver.transporter);
        console.log(chalk_1.default.green('Connection to mail server established.'));
    }
    catch (e) {
        console.log(e);
        console.error(chalk_1.default.red('Error establishing connection to the mail server.'));
    }
});
exports.connectMailDriver = connectMailDriver;
const sendMail = (options) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        if (!(mailDriver === null || mailDriver === void 0 ? void 0 : mailDriver.transporter)) {
            logger_1.default.error('Email provider not set');
            return;
        }
        options = yield hooks_1.default.applyFilters('mail/send', options);
        const result = yield mailDriver.transporter.sendMail(options);
        yield hooks_1.default.doAction('mail/dispatched', result);
    }
    catch (err) {
        logger_1.default.error('Failed to send email', err);
    }
});
exports.sendMail = sendMail;
