"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const typeorm_1 = require("typeorm");
const user_meta_model_1 = __importDefault(require("./user-meta.model"));
const user_session_model_1 = __importDefault(require("./user-session.model"));
const user_token_model_1 = __importDefault(require("./user-token.model"));
const model_1 = require("../../shared/interfaces/model");
const group_model_1 = __importDefault(require("./group.model"));
const lodash_1 = __importDefault(require("lodash"));
const crypto_1 = require("../common/crypto");
let User = class User extends typeorm_1.BaseEntity {
    sanitizeEmail() {
        if (this.email) {
            this.email = this.email.trim().toLowerCase();
        }
    }
    applyMeta(meta) {
        const metaObject = lodash_1.default.keyBy(this.meta, 'key');
        Object.keys(meta !== null && meta !== void 0 ? meta : {}).forEach((key) => {
            const value = meta === null || meta === void 0 ? void 0 : meta[key];
            if (value === undefined) {
                return;
            }
            if (metaObject === null || metaObject === void 0 ? void 0 : metaObject[key]) {
                metaObject[key].value = value;
            }
            else {
                metaObject[key] = { key, value };
            }
        });
        this.meta = lodash_1.default.values(metaObject);
    }
    comparePasswordSync(password) {
        return (0, crypto_1.compareSync)(password, this.password);
    }
    comparePassword(password) {
        return __awaiter(this, void 0, void 0, function* () {
            return (0, crypto_1.compare)(password, this.password);
        });
    }
    setPasswordSync(password) {
        this.password = (0, crypto_1.hashSync)(password);
    }
    setPassword(password) {
        return __awaiter(this, void 0, void 0, function* () {
            this.password = yield (0, crypto_1.hash)(password);
        });
    }
};
__decorate([
    (0, typeorm_1.PrimaryGeneratedColumn)('increment'),
    __metadata("design:type", Number)
], User.prototype, "id", void 0);
__decorate([
    (0, typeorm_1.Column)(),
    (0, typeorm_1.Index)({ unique: true }),
    __metadata("design:type", String)
], User.prototype, "email", void 0);
__decorate([
    (0, typeorm_1.Column)({ select: false }),
    __metadata("design:type", String)
], User.prototype, "password", void 0);
__decorate([
    (0, typeorm_1.Column)({ nullable: true }),
    __metadata("design:type", String)
], User.prototype, "firstName", void 0);
__decorate([
    (0, typeorm_1.Column)({ nullable: true }),
    __metadata("design:type", String)
], User.prototype, "lastName", void 0);
__decorate([
    (0, typeorm_1.Column)({ length: 64, type: 'varchar', default: model_1.UserStatus.ACTIVE }),
    __metadata("design:type", String)
], User.prototype, "status", void 0);
__decorate([
    (0, typeorm_1.CreateDateColumn)(),
    __metadata("design:type", Date)
], User.prototype, "createdAt", void 0);
__decorate([
    (0, typeorm_1.UpdateDateColumn)(),
    __metadata("design:type", Date)
], User.prototype, "updatedAt", void 0);
__decorate([
    (0, typeorm_1.ManyToMany)(() => group_model_1.default, (group) => group.users, {
        cascade: true,
        onDelete: 'CASCADE'
    }),
    __metadata("design:type", Array)
], User.prototype, "groups", void 0);
__decorate([
    (0, typeorm_1.OneToMany)(() => user_meta_model_1.default, (userMeta) => userMeta.user, { cascade: true }),
    __metadata("design:type", Array)
], User.prototype, "meta", void 0);
__decorate([
    (0, typeorm_1.OneToMany)(() => user_session_model_1.default, (session) => session.user, { cascade: true }),
    __metadata("design:type", Array)
], User.prototype, "sessions", void 0);
__decorate([
    (0, typeorm_1.OneToMany)(() => user_token_model_1.default, (userToken) => userToken.user, { cascade: true }),
    __metadata("design:type", Array)
], User.prototype, "tokens", void 0);
__decorate([
    (0, typeorm_1.BeforeInsert)(),
    (0, typeorm_1.BeforeUpdate)(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], User.prototype, "sanitizeEmail", null);
User = __decorate([
    (0, typeorm_1.Entity)()
], User);
exports.default = User;
