"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.launch = void 0;
const express_1 = __importDefault(require("express"));
const chalk_1 = __importDefault(require("chalk"));
const api_1 = require("./api");
const hooks_1 = __importDefault(require("./hooks"));
const launch = () => __awaiter(void 0, void 0, void 0, function* () {
    try {
        yield hooks_1.default.doAction('core/init');
        const app = (0, express_1.default)();
        const apiApp = yield (0, api_1.launchApi)();
        app.use('/api', apiApp);
        yield hooks_1.default.doAction('server/init', app);
        app.use((_req, res) => __awaiter(void 0, void 0, void 0, function* () {
            const response = yield hooks_1.default.applyFilters('server/notFound', res.status(404).json({ message: 'not_found' }));
            response.send();
        }));
        const port = process.env.SERVER_PORT || 4000;
        const host = process.env.SERVER_HOST || 'http://localhost';
        app.listen(port, () => {
            console.log(chalk_1.default.green(`Express started on ${port} port.`));
            console.log(chalk_1.default.green(`To open admin visit ${host}:${port}/admin`));
        }).on('error', (e) => {
            console.log(e);
            console.log(chalk_1.default.red('Error starting Express'));
        });
    }
    catch (e) {
        console.log(e);
        console.log(chalk_1.default.red('Error starting Express'));
    }
});
exports.launch = launch;
